/**************************************************************************
 *
 *   Copyright (c) 2002 Openwave Systems Inc.  All rights reserved.
 *
 *   Openwave Systems Inc. grants you the right to use the Openwave Location
 *   Studio SDK software and its related documentation only according
 *   to Openwave's Location Studio SDK End User Software License Agreement.
 *
 *   Openwave and the Openwave logo are registered trademarks of Openwave
 *   Systems Inc. All other trademarks and registered trademarks are the
 *   properties of their respective owners.
 *
 **************************************************************************
 * $Header: /development/WLS/CVS/SDKLSt/MethodA/src/LstDemoServlet.java,v 1.2 2002/12/02 20:53:39 rreynold Exp $
 **************************************************************************/

import java.io.FileInputStream;
import java.io.IOException;

import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.signalsoftcorp.lst.client.SubscriberId;
import com.signalsoftcorp.lst.client.WLILocation;
import com.signalsoftcorp.lst.client.WLP10Constants;
import com.signalsoftcorp.lst.client.WLP10LocationRequest;
import com.signalsoftcorp.lst.client.WLP10LocationResponse;
import com.signalsoftcorp.lst.client.WLP10Position;
import com.signalsoftcorp.lst.client.WMIMessaging;
import com.signalsoftcorp.lst.client.WMP10Constants;
import com.signalsoftcorp.lst.client.WMP10MessagingRequest;
import com.signalsoftcorp.lst.client.WMP10MessagingResponse;
import com.signalsoftcorp.lst.client.WMP10SmsRequest;
import electric.registry.Registry;

/**
 * This is an example of a location based wireless application 
 * using the services offered by Location Studio.
 * A subscriber sends an SMS message containing a command.
 * The command 'GAS' means 'Send me an SMS with directions to nearest gas station'
 * The command 'RESTAURANT' means 'Send me an SMS with directions to nearest restaurant'
 */
public class LstDemoServlet extends HttpServlet
{
    // Command types
    private static final int NONE = -1;
    private static final int RESTAURANT = 0;
    private static final int GAS = 1;
    
    // Location request constants
    
    // Maximum age for the location of the subscriber
    private static final int MAX_LOC_AGE = 60*5;
    // Maximum time to wait for a location
    private static final int TIMEOUT = 60;
    
    // The Web services
    private WLILocation locationService = null;
    private WMIMessaging messagingService = null;
    
    // Client application id and password
    private String clientId = null;
    private String password = null;
            
    public void init() throws ServletException{
     try 
     {
        // Get the variables of this interface from the example_variables.properties file.
        // Use a command line argument to specify a different properties file.
        String property_filename = "example_variables.properties";

        FileInputStream property_file = new FileInputStream( property_filename );
        Properties variables = new Properties();
        variables.load( property_file );

        // The URL to the AAI webservice
        String url = variables.getProperty("LStURL");
        clientId = variables.getProperty("clientId");
        password = variables.getProperty("password");
        
        // Bind to AAI Web Services 
        try {
            messagingService = (WMIMessaging)Registry.bind(url, WMIMessaging.class);
            locationService = (WLILocation)Registry.bind(url, WLILocation.class);            
        }
        catch(Exception e) {
            throw new ServletException("Failed to bind to AAI services",e);
        }
     }
     catch(Exception e) {
        throw new ServletException("Failed to read property file",e);
     }
    }

    /**
     * Handles MO SMS from LocationStudio
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException
    {
        WLP10LocationRequest locationReq;
        WLP10LocationResponse locationResp;
        
        // Exctract incoming SMS parameters
        
        // Retrieve temporary subscriber ID, TSID
        String sid = req.getParameter("TSID");
        // The SMS message body
        String message = req.getParameter("message");
        
        // Create a subscriber id that we will use to reference this subscriber in calls to AAI
        SubscriberId subscriber = new SubscriberId("TSID",sid);
        
        // Get the location for this subscriber   
        locationReq = new WLP10LocationRequest(new SubscriberId[] {subscriber},TIMEOUT,MAX_LOC_AGE);
        locationResp = locationService.getLocation(clientId,password,locationReq);
        WLP10Position pos = locationResp.getWLP10Position()[0];
        
        // Make sure the request succeeded
        if(pos.getResponseCode()==WLP10Constants.OK) {
            // Search for the nearest object given the position and the command in the SMS message body
            String responseMessage = search(message,locationResp.getWLP10Position()[0]);
            // Send a response SMS with directions to the object
            sendSmsResponse(subscriber,responseMessage);
        }
        else {
            sendSmsResponse(subscriber,"Could not locate you. Sorry");
        }
    }
    
    /**
     * Sends an SMS message to the given subscriber
     */
    private final void sendSmsResponse(SubscriberId subscriberId,String message) {
        WMP10SmsRequest smsReq = new WMP10SmsRequest(message,subscriberId);
        WMP10MessagingResponse messagingResponse = messagingService.sendSmsMessage(clientId,password,smsReq);
        if(messagingResponse.getResponseCode()!=WMP10Constants.OK) {
            log("Failed to send SMS message: "+messagingResponse.toString());
        }
    }
    
    /**
     * Mockup for a method that looks in some database for the nearest
     * object given a location.
     */
    private String search(String message,WLP10Position pos) {
        int type = parseMessage(message);
        String latitude = pos.getLatitude();
        String longitude = pos.getLongitude();
        
        // Here we would search our database
        
        if(type==RESTAURANT) {
            return "TONY'S RESTAURANT , 0.3 miles SW on E22";    
        }
        else if(type==GAS) {
            return "MIKES GAS, 1.2 miles N on E22";
        }
        else {
            return "I don't understand your command: "+message;
        }
    }
    
    /**
     * Parses the SMS message body for command keywords
     */
    private int parseMessage(String s) {
        if(s.startsWith("restaurant") || s.startsWith("Restaurant") || s.startsWith("RESTAURANT") ) {
            return RESTAURANT;
        }
        else if(s.startsWith("gas") || s.startsWith("Gas") || s.startsWith("GAS") ) {
            return GAS;
        }
        else {
            return NONE;
        }
    }
}
